# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .archives import (
    ArchivesResource,
    AsyncArchivesResource,
    ArchivesResourceWithRawResponse,
    AsyncArchivesResourceWithRawResponse,
    ArchivesResourceWithStreamingResponse,
    AsyncArchivesResourceWithStreamingResponse,
)
from .passages import (
    PassagesResource,
    AsyncPassagesResource,
    PassagesResourceWithRawResponse,
    AsyncPassagesResourceWithRawResponse,
    PassagesResourceWithStreamingResponse,
    AsyncPassagesResourceWithStreamingResponse,
)

__all__ = [
    "PassagesResource",
    "AsyncPassagesResource",
    "PassagesResourceWithRawResponse",
    "AsyncPassagesResourceWithRawResponse",
    "PassagesResourceWithStreamingResponse",
    "AsyncPassagesResourceWithStreamingResponse",
    "ArchivesResource",
    "AsyncArchivesResource",
    "ArchivesResourceWithRawResponse",
    "AsyncArchivesResourceWithRawResponse",
    "ArchivesResourceWithStreamingResponse",
    "AsyncArchivesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["AgentExportFileParams"]


class AgentExportFileParams(TypedDict, total=False):
    max_steps: int

    use_legacy_format: bool
    """
    If True, exports using the legacy single-agent 'v1' format with inline
    tools/blocks. If False, exports using the new multi-entity 'v2' format, with
    separate agents, tools, blocks, files, etc.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["EmbeddingConfigParam"]


class EmbeddingConfigParam(TypedDict, total=False):
    embedding_dim: Required[int]
    """The dimension of the embedding."""

    embedding_endpoint_type: Required[
        Literal[
            "openai",
            "anthropic",
            "bedrock",
            "google_ai",
            "google_vertex",
            "azure",
            "groq",
            "ollama",
            "webui",
            "webui-legacy",
            "lmstudio",
            "lmstudio-legacy",
            "llamacpp",
            "koboldcpp",
            "vllm",
            "hugging-face",
            "mistral",
            "together",
            "pinecone",
        ]
    ]
    """The endpoint type for the model."""

    embedding_model: Required[str]
    """The model for the embedding."""

    azure_deployment: Optional[str]
    """The Azure deployment for the model."""

    azure_endpoint: Optional[str]
    """The Azure endpoint for the model."""

    azure_version: Optional[str]
    """The Azure version for the model."""

    batch_size: int
    """The maximum batch size for processing embeddings."""

    embedding_chunk_size: Optional[int]
    """The chunk size of the embedding."""

    embedding_endpoint: Optional[str]
    """The endpoint for the model (`None` if local)."""

    handle: Optional[str]
    """The handle for this config, in the format provider/model-name."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["AgentCreateResponse"]


class AgentCreateResponse(BaseModel):
    agent_ids: List[str]
    """Array of created agent IDs"""

    deployment_id: str
    """The deployment ID for the created agents"""

    group_id: Optional[str] = None
    """Optional group ID if agents were created in a group"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.provider_trace import ProviderTrace

__all__ = ["TraceResource", "AsyncTraceResource"]


class TraceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TraceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return TraceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TraceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return TraceResourceWithStreamingResponse(self)

    def retrieve(
        self,
        step_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProviderTrace]:
        """
        Retrieve Trace For Step

        Args:
          step_id: The ID of the step in the format 'step-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return self._get(
            f"/v1/steps/{step_id}/trace",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProviderTrace,
        )


class AsyncTraceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTraceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTraceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTraceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncTraceResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        step_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProviderTrace]:
        """
        Retrieve Trace For Step

        Args:
          step_id: The ID of the step in the format 'step-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return await self._get(
            f"/v1/steps/{step_id}/trace",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProviderTrace,
        )


class TraceResourceWithRawResponse:
    def __init__(self, trace: TraceResource) -> None:
        self._trace = trace

        self.retrieve = to_raw_response_wrapper(
            trace.retrieve,
        )


class AsyncTraceResourceWithRawResponse:
    def __init__(self, trace: AsyncTraceResource) -> None:
        self._trace = trace

        self.retrieve = async_to_raw_response_wrapper(
            trace.retrieve,
        )


class TraceResourceWithStreamingResponse:
    def __init__(self, trace: TraceResource) -> None:
        self._trace = trace

        self.retrieve = to_streamed_response_wrapper(
            trace.retrieve,
        )


class AsyncTraceResourceWithStreamingResponse:
    def __init__(self, trace: AsyncTraceResource) -> None:
        self._trace = trace

        self.retrieve = async_to_streamed_response_wrapper(
            trace.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ApprovalReturnParam"]


class ApprovalReturnParam(TypedDict, total=False):
    approve: Required[bool]
    """Whether the tool has been approved"""

    tool_call_id: Required[str]
    """The ID of the tool call that corresponds to this approval"""

    reason: Optional[str]
    """An optional explanation for the provided approval status"""

    type: Literal["approval"]
    """The message type to be created."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing
import inspect
from typing import Dict, Iterable, Optional
from textwrap import dedent
from typing_extensions import Literal

import httpx
from pydantic import BaseModel

from ..types import tool_list_params, tool_create_params, tool_update_params, tool_upsert_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncArrayPage, AsyncArrayPage
from ..types.tool import Tool, BaseTool
from .._base_client import AsyncPaginator, make_request_options
from ..types.npm_requirement_param import NpmRequirementParam
from ..types.pip_requirement_param import PipRequirementParam
from ..types.tool_upsert_base_tools_response import ToolUpsertBaseToolsResponse

__all__ = ["ToolsResource", "AsyncToolsResource"]


class ToolsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return ToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return ToolsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        source_code: str,
        args_json_schema: Optional[Dict[str, object]] | Omit = omit,
        default_requires_approval: Optional[bool] | Omit = omit,
        description: Optional[str] | Omit = omit,
        enable_parallel_execution: Optional[bool] | Omit = omit,
        json_schema: Optional[Dict[str, object]] | Omit = omit,
        npm_requirements: Optional[Iterable[NpmRequirementParam]] | Omit = omit,
        pip_requirements: Optional[Iterable[PipRequirementParam]] | Omit = omit,
        return_char_limit: int | Omit = omit,
        source_type: str | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Create a new tool

        Args:
          source_code: The source code of the function.

          args_json_schema: The args JSON schema of the function.

          default_requires_approval: Whether or not to require approval before executing this tool.

          description: The description of the tool.

          enable_parallel_execution: If set to True, then this tool will potentially be executed concurrently with
              other tools. Default False.

          json_schema: The JSON schema of the function (auto-generated from source_code if not
              provided)

          npm_requirements: Optional list of npm packages required by this tool.

          pip_requirements: Optional list of pip packages required by this tool.

          return_char_limit: The maximum number of characters in the response.

          source_type: The source type of the function.

          tags: Metadata tags.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/tools/",
            body=maybe_transform(
                {
                    "source_code": source_code,
                    "args_json_schema": args_json_schema,
                    "default_requires_approval": default_requires_approval,
                    "description": description,
                    "enable_parallel_execution": enable_parallel_execution,
                    "json_schema": json_schema,
                    "npm_requirements": npm_requirements,
                    "pip_requirements": pip_requirements,
                    "return_char_limit": return_char_limit,
                    "source_type": source_type,
                    "tags": tags,
                },
                tool_create_params.ToolCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tool,
        )

    def retrieve(
        self,
        tool_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Get a tool by ID

        Args:
          tool_id: The ID of the tool in the format 'tool-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return self._get(
            f"/v1/tools/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tool,
        )

    def update(
        self,
        tool_id: str,
        *,
        args_json_schema: Optional[Dict[str, object]] | Omit = omit,
        default_requires_approval: Optional[bool] | Omit = omit,
        description: Optional[str] | Omit = omit,
        enable_parallel_execution: Optional[bool] | Omit = omit,
        json_schema: Optional[Dict[str, object]] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        npm_requirements: Optional[Iterable[NpmRequirementParam]] | Omit = omit,
        pip_requirements: Optional[Iterable[PipRequirementParam]] | Omit = omit,
        return_char_limit: Optional[int] | Omit = omit,
        source_code: Optional[str] | Omit = omit,
        source_type: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Update an existing tool

        Args:
          tool_id: The ID of the tool in the format 'tool-<uuid4>'

          args_json_schema: The args JSON schema of the function.

          default_requires_approval: Whether or not to require approval before executing this tool.

          description: The description of the tool.

          enable_parallel_execution: If set to True, then this tool will potentially be executed concurrently with
              other tools. Default False.

          json_schema: The JSON schema of the function (auto-generated from source_code if not
              provided)

          metadata: A dictionary of additional metadata for the tool.

          npm_requirements: Optional list of npm packages required by this tool.

          pip_requirements: Optional list of pip packages required by this tool.

          return_char_limit: The maximum number of characters in the response.

          source_code: The source code of the function.

          source_type: The type of the source code.

          tags: Metadata tags.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return self._patch(
            f"/v1/tools/{tool_id}",
            body=maybe_transform(
                {
                    "args_json_schema": args_json_schema,
                    "default_requires_approval": default_requires_approval,
                    "description": description,
                    "enable_parallel_execution": enable_parallel_execution,
                    "json_schema": json_schema,
                    "metadata": metadata,
                    "npm_requirements": npm_requirements,
                    "pip_requirements": pip_requirements,
                    "return_char_limit": return_char_limit,
                    "source_code": source_code,
                    "source_type": source_type,
                    "tags": tags,
                },
                tool_update_params.ToolUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tool,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        exclude_tool_types: Optional[SequenceNotStr[str]] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        name: Optional[str] | Omit = omit,
        names: Optional[SequenceNotStr[str]] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        return_only_letta_tools: Optional[bool] | Omit = omit,
        search: Optional[str] | Omit = omit,
        tool_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        tool_types: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncArrayPage[Tool]:
        """
        Get a list of all tools available to agents.

        Args:
          after: Tool ID cursor for pagination. Returns tools that come after this tool ID in the
              specified sort order

          before: Tool ID cursor for pagination. Returns tools that come before this tool ID in
              the specified sort order

          exclude_tool_types: Tool type(s) to exclude - accepts repeated params or comma-separated values

          limit: Maximum number of tools to return

          name: Filter by single tool name

          names: Filter by specific tool names

          order: Sort order for tools by creation time. 'asc' for oldest first, 'desc' for newest
              first

          order_by: Field to sort by

          return_only_letta_tools: Return only tools with tool*type starting with 'letta*'

          search: Search tool names (case-insensitive partial match)

          tool_ids: Filter by specific tool IDs - accepts repeated params or comma-separated values

          tool_types: Filter by tool type(s) - accepts repeated params or comma-separated values

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/tools/",
            page=SyncArrayPage[Tool],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "exclude_tool_types": exclude_tool_types,
                        "limit": limit,
                        "name": name,
                        "names": names,
                        "order": order,
                        "order_by": order_by,
                        "return_only_letta_tools": return_only_letta_tools,
                        "search": search,
                        "tool_ids": tool_ids,
                        "tool_types": tool_types,
                    },
                    tool_list_params.ToolListParams,
                ),
            ),
            model=Tool,
        )

    def delete(
        self,
        tool_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a tool by name

        Args:
          tool_id: The ID of the tool in the format 'tool-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return self._delete(
            f"/v1/tools/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def upsert(
        self,
        *,
        source_code: str,
        args_json_schema: Optional[Dict[str, object]] | Omit = omit,
        default_requires_approval: Optional[bool] | Omit = omit,
        description: Optional[str] | Omit = omit,
        enable_parallel_execution: Optional[bool] | Omit = omit,
        json_schema: Optional[Dict[str, object]] | Omit = omit,
        npm_requirements: Optional[Iterable[NpmRequirementParam]] | Omit = omit,
        pip_requirements: Optional[Iterable[PipRequirementParam]] | Omit = omit,
        return_char_limit: int | Omit = omit,
        source_type: str | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Create or update a tool

        Args:
          source_code: The source code of the function.

          args_json_schema: The args JSON schema of the function.

          default_requires_approval: Whether or not to require approval before executing this tool.

          description: The description of the tool.

          enable_parallel_execution: If set to True, then this tool will potentially be executed concurrently with
              other tools. Default False.

          json_schema: The JSON schema of the function (auto-generated from source_code if not
              provided)

          npm_requirements: Optional list of npm packages required by this tool.

          pip_requirements: Optional list of pip packages required by this tool.

          return_char_limit: The maximum number of characters in the response.

          source_type: The source type of the function.

          tags: Metadata tags.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._put(
            "/v1/tools/",
            body=maybe_transform(
                {
                    "source_code": source_code,
                    "args_json_schema": args_json_schema,
                    "default_requires_approval": default_requires_approval,
                    "description": description,
                    "enable_parallel_execution": enable_parallel_execution,
                    "json_schema": json_schema,
                    "npm_requirements": npm_requirements,
                    "pip_requirements": pip_requirements,
                    "return_char_limit": return_char_limit,
                    "source_type": source_type,
                    "tags": tags,
                },
                tool_upsert_params.ToolUpsertParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tool,
        )

    def upsert_base_tools(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolUpsertBaseToolsResponse:
        """Upsert base tools"""
        return self._post(
            "/v1/tools/add-base-tools",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolUpsertBaseToolsResponse,
        )

    def create_from_function(
        self,
        *,
        func: typing.Callable[..., typing.Any],
        args_schema: typing.Optional[typing.Type[BaseModel]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        source_type: str | Omit = omit,
        json_schema: Optional[Dict[str, object]] | Omit = omit,
        return_char_limit: int | Omit = omit,
        pip_requirements: Optional[Iterable[PipRequirementParam]] | Omit = omit,
        npm_requirements: Optional[Iterable[NpmRequirementParam]] | Omit = omit,
        default_requires_approval: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Create a new tool from a callable

        Args:
          func: The callable to create the tool from.

          args_schema: The arguments schema of the function, as a Pydantic model.

          description: The description of the tool.

          tags: Metadata tags.

          source_type: The source type of the function.

          json_schema: The JSON schema of the function (auto-generated from source_code if not
              provided)

          return_char_limit: The maximum number of characters in the response.

          pip_requirements: Optional list of pip packages required by this tool.

          npm_requirements: Optional list of npm packages required by this tool.

          default_requires_approval: Whether or not to require approval before executing this tool.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

        Examples:
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        
        def add_two_numbers(a: int, b: int) -> int:
            return a + b
        
        client.tools.create_from_function(
            func=add_two_numbers,
        )

        class InventoryEntryData(BaseModel):
            data: InventoryEntry
            quantity_change: int

        def manage_inventory(data: InventoryEntry, quantity_change: int) -> bool:
            pass
        
        client.tools.create_from_function(
            func=manage_inventory,
            args_schema=InventoryEntryData,
        )
        """
        source_code = dedent(inspect.getsource(func))
        args_json_schema: Optional[Dict[str, object]] | Omit = omit
        if not isinstance(args_schema, Omit) and args_schema is not None:
            args_json_schema = args_schema.model_json_schema()

        return self.create(
            source_code=source_code,
            args_json_schema=args_json_schema,
            description=description,
            tags=tags,
            source_type=source_type,
            json_schema=json_schema,
            return_char_limit=return_char_limit,
            pip_requirements=pip_requirements,
            npm_requirements=npm_requirements,
            default_requires_approval=default_requires_approval,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    def upsert_from_function(
        self,
        *,
        func: typing.Callable[..., typing.Any],
        args_schema: typing.Optional[typing.Type[BaseModel]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        source_type: str | Omit = omit,
        json_schema: Optional[Dict[str, object]] | Omit = omit,
        return_char_limit: int | Omit = omit,
        pip_requirements: Optional[Iterable[PipRequirementParam]] | Omit = omit,
        npm_requirements: Optional[Iterable[NpmRequirementParam]] | Omit = omit,
        default_requires_approval: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Create or update a tool from a callable

        Args:
          func: The callable to create or update the tool from.

          args_schema: The arguments schema of the function, as a Pydantic model.

          description: The description of the tool.

          tags: Metadata tags.

          source_type: The source type of the function.

          json_schema: The JSON schema of the function (auto-generated from source_code if not
              provided)

          return_char_limit: The maximum number of characters in the response.

          pip_requirements: Optional list of pip packages required by this tool.

          npm_requirements: Optional list of npm packages required by this tool.

          default_requires_approval: Whether or not to require approval before executing this tool.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

        Examples:
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )

        def add_two_numbers(a: int, b: int) -> int:
            return a + b
        
        client.tools.upsert_from_function(
            func=add_two_numbers,
        )

        class InventoryEntryData(BaseModel):
            data: InventoryEntry
            quantity_change: int

        def manage_inventory(data: InventoryEntry, quantity_change: int) -> bool:
            pass
        
        client.tools.upsert_from_function(
            func=manage_inventory,
            args_schema=InventoryEntryData,
        )
        """
        source_code = dedent(inspect.getsource(func))
        args_json_schema: Optional[Dict[str, object]] | Omit = omit
        if not isinstance(args_schema, Omit) and args_schema is not None:
            args_json_schema = args_schema.model_json_schema()

        return self.upsert(
            source_code=source_code,
            args_json_schema=args_json_schema,
            description=description,
            tags=tags,
            source_type=source_type,
            json_schema=json_schema,
            return_char_limit=return_char_limit,
            pip_requirements=pip_requirements,
            npm_requirements=npm_requirements,
            default_requires_approval=default_requires_approval,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    def add(
        self,
        *,
        tool: BaseTool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Add a tool to Letta from a custom Tool class

        Args:
          tool: The tool object to be added.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

        Examples:
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )

        class InventoryItem(BaseModel):
            sku: str  # Unique product identifier
            name: str  # Product name
            price: float  # Current price
            category: str  # Product category (e.g., "Electronics", "Clothing")

        class InventoryEntry(BaseModel):
            timestamp: int  # Unix timestamp of the transaction
            item: InventoryItem  # The product being updated
            transaction_id: str  # Unique identifier for this inventory update

        class InventoryEntryData(BaseModel):
            data: InventoryEntry
            quantity_change: int  # Change in quantity (positive for additions, negative for removals)

        class ManageInventoryTool(BaseTool):
            name: str = "manage_inventory"
            args_schema: Type[BaseModel] = InventoryEntryData
            description: str = "Update inventory catalogue with a new data entry"
            tags: List[str] = ["inventory", "shop"]

            def run(self, data: InventoryEntry, quantity_change: int) -> bool:
                '''
                Implementation of the manage_inventory tool
                '''
                print(f"Updated inventory for {data.item.name} with a quantity change of {quantity_change}")
                return True
                
        client.tools.add(
            tool=ManageInventoryTool()
        )
        """
        source_code = tool.get_source_code()
        args_json_schema = tool.args_schema.model_json_schema() if tool.args_schema else None

        # Convert PipRequirement/NpmRequirement models to Param dicts
        pip_requirements_param = (
            [typing.cast(PipRequirementParam, req.model_dump()) for req in tool.pip_requirements]
            if tool.pip_requirements
            else omit
        )
        npm_requirements_param = (
            [typing.cast(NpmRequirementParam, req.model_dump()) for req in tool.npm_requirements]
            if tool.npm_requirements
            else omit
        )

        return self.upsert(
            source_code=source_code,
            args_json_schema=args_json_schema or omit,
            description=tool.description or omit,
            tags=tool.tags or omit,
            source_type=tool.source_type or omit,
            json_schema=tool.json_schema or omit,
            return_char_limit=tool.return_char_limit or omit,
            pip_requirements=pip_requirements_param,
            npm_requirements=npm_requirements_param,
            default_requires_approval=tool.default_requires_approval or omit,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

class AsyncToolsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncToolsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        source_code: str,
        args_json_schema: Optional[Dict[str, object]] | Omit = omit,
        default_requires_approval: Optional[bool] | Omit = omit,
        description: Optional[str] | Omit = omit,
        enable_parallel_execution: Optional[bool] | Omit = omit,
        json_schema: Optional[Dict[str, object]] | Omit = omit,
        npm_requirements: Optional[Iterable[NpmRequirementParam]] | Omit = omit,
        pip_requirements: Optional[Iterable[PipRequirementParam]] | Omit = omit,
        return_char_limit: int | Omit = omit,
        source_type: str | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Create a new tool

        Args:
          source_code: The source code of the function.

          args_json_schema: The args JSON schema of the function.

          default_requires_approval: Whether or not to require approval before executing this tool.

          description: The description of the tool.

          enable_parallel_execution: If set to True, then this tool will potentially be executed concurrently with
              other tools. Default False.

          json_schema: The JSON schema of the function (auto-generated from source_code if not
              provided)

          npm_requirements: Optional list of npm packages required by this tool.

          pip_requirements: Optional list of pip packages required by this tool.

          return_char_limit: The maximum number of characters in the response.

          source_type: The source type of the function.

          tags: Metadata tags.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/tools/",
            body=await async_maybe_transform(
                {
                    "source_code": source_code,
                    "args_json_schema": args_json_schema,
                    "default_requires_approval": default_requires_approval,
                    "description": description,
                    "enable_parallel_execution": enable_parallel_execution,
                    "json_schema": json_schema,
                    "npm_requirements": npm_requirements,
                    "pip_requirements": pip_requirements,
                    "return_char_limit": return_char_limit,
                    "source_type": source_type,
                    "tags": tags,
                },
                tool_create_params.ToolCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tool,
        )

    async def retrieve(
        self,
        tool_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Get a tool by ID

        Args:
          tool_id: The ID of the tool in the format 'tool-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return await self._get(
            f"/v1/tools/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tool,
        )

    async def update(
        self,
        tool_id: str,
        *,
        args_json_schema: Optional[Dict[str, object]] | Omit = omit,
        default_requires_approval: Optional[bool] | Omit = omit,
        description: Optional[str] | Omit = omit,
        enable_parallel_execution: Optional[bool] | Omit = omit,
        json_schema: Optional[Dict[str, object]] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        npm_requirements: Optional[Iterable[NpmRequirementParam]] | Omit = omit,
        pip_requirements: Optional[Iterable[PipRequirementParam]] | Omit = omit,
        return_char_limit: Optional[int] | Omit = omit,
        source_code: Optional[str] | Omit = omit,
        source_type: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Update an existing tool

        Args:
          tool_id: The ID of the tool in the format 'tool-<uuid4>'

          args_json_schema: The args JSON schema of the function.

          default_requires_approval: Whether or not to require approval before executing this tool.

          description: The description of the tool.

          enable_parallel_execution: If set to True, then this tool will potentially be executed concurrently with
              other tools. Default False.

          json_schema: The JSON schema of the function (auto-generated from source_code if not
              provided)

          metadata: A dictionary of additional metadata for the tool.

          npm_requirements: Optional list of npm packages required by this tool.

          pip_requirements: Optional list of pip packages required by this tool.

          return_char_limit: The maximum number of characters in the response.

          source_code: The source code of the function.

          source_type: The type of the source code.

          tags: Metadata tags.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return await self._patch(
            f"/v1/tools/{tool_id}",
            body=await async_maybe_transform(
                {
                    "args_json_schema": args_json_schema,
                    "default_requires_approval": default_requires_approval,
                    "description": description,
                    "enable_parallel_execution": enable_parallel_execution,
                    "json_schema": json_schema,
                    "metadata": metadata,
                    "npm_requirements": npm_requirements,
                    "pip_requirements": pip_requirements,
                    "return_char_limit": return_char_limit,
                    "source_code": source_code,
                    "source_type": source_type,
                    "tags": tags,
                },
                tool_update_params.ToolUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tool,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        exclude_tool_types: Optional[SequenceNotStr[str]] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        name: Optional[str] | Omit = omit,
        names: Optional[SequenceNotStr[str]] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        return_only_letta_tools: Optional[bool] | Omit = omit,
        search: Optional[str] | Omit = omit,
        tool_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        tool_types: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Tool, AsyncArrayPage[Tool]]:
        """
        Get a list of all tools available to agents.

        Args:
          after: Tool ID cursor for pagination. Returns tools that come after this tool ID in the
              specified sort order

          before: Tool ID cursor for pagination. Returns tools that come before this tool ID in
              the specified sort order

          exclude_tool_types: Tool type(s) to exclude - accepts repeated params or comma-separated values

          limit: Maximum number of tools to return

          name: Filter by single tool name

          names: Filter by specific tool names

          order: Sort order for tools by creation time. 'asc' for oldest first, 'desc' for newest
              first

          order_by: Field to sort by

          return_only_letta_tools: Return only tools with tool*type starting with 'letta*'

          search: Search tool names (case-insensitive partial match)

          tool_ids: Filter by specific tool IDs - accepts repeated params or comma-separated values

          tool_types: Filter by tool type(s) - accepts repeated params or comma-separated values

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/tools/",
            page=AsyncArrayPage[Tool],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "exclude_tool_types": exclude_tool_types,
                        "limit": limit,
                        "name": name,
                        "names": names,
                        "order": order,
                        "order_by": order_by,
                        "return_only_letta_tools": return_only_letta_tools,
                        "search": search,
                        "tool_ids": tool_ids,
                        "tool_types": tool_types,
                    },
                    tool_list_params.ToolListParams,
                ),
            ),
            model=Tool,
        )

    async def delete(
        self,
        tool_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a tool by name

        Args:
          tool_id: The ID of the tool in the format 'tool-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return await self._delete(
            f"/v1/tools/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def upsert(
        self,
        *,
        source_code: str,
        args_json_schema: Optional[Dict[str, object]] | Omit = omit,
        default_requires_approval: Optional[bool] | Omit = omit,
        description: Optional[str] | Omit = omit,
        enable_parallel_execution: Optional[bool] | Omit = omit,
        json_schema: Optional[Dict[str, object]] | Omit = omit,
        npm_requirements: Optional[Iterable[NpmRequirementParam]] | Omit = omit,
        pip_requirements: Optional[Iterable[PipRequirementParam]] | Omit = omit,
        return_char_limit: int | Omit = omit,
        source_type: str | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Create or update a tool

        Args:
          source_code: The source code of the function.

          args_json_schema: The args JSON schema of the function.

          default_requires_approval: Whether or not to require approval before executing this tool.

          description: The description of the tool.

          enable_parallel_execution: If set to True, then this tool will potentially be executed concurrently with
              other tools. Default False.

          json_schema: The JSON schema of the function (auto-generated from source_code if not
              provided)

          npm_requirements: Optional list of npm packages required by this tool.

          pip_requirements: Optional list of pip packages required by this tool.

          return_char_limit: The maximum number of characters in the response.

          source_type: The source type of the function.

          tags: Metadata tags.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._put(
            "/v1/tools/",
            body=await async_maybe_transform(
                {
                    "source_code": source_code,
                    "args_json_schema": args_json_schema,
                    "default_requires_approval": default_requires_approval,
                    "description": description,
                    "enable_parallel_execution": enable_parallel_execution,
                    "json_schema": json_schema,
                    "npm_requirements": npm_requirements,
                    "pip_requirements": pip_requirements,
                    "return_char_limit": return_char_limit,
                    "source_type": source_type,
                    "tags": tags,
                },
                tool_upsert_params.ToolUpsertParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tool,
        )

    async def upsert_base_tools(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolUpsertBaseToolsResponse:
        """Upsert base tools"""
        return await self._post(
            "/v1/tools/add-base-tools",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolUpsertBaseToolsResponse,
        )

    async def create_from_function(
        self,
        *,
        func: typing.Callable[..., typing.Any],
        args_schema: typing.Optional[typing.Type[BaseModel]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        source_type: str | Omit = omit,
        json_schema: Optional[Dict[str, object]] | Omit = omit,
        return_char_limit: int | Omit = omit,
        pip_requirements: Optional[Iterable[PipRequirementParam]] | Omit = omit,
        npm_requirements: Optional[Iterable[NpmRequirementParam]] | Omit = omit,
        default_requires_approval: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Create a new tool from a callable

        Args:
          func: The callable to create the tool from.

          args_schema: The arguments schema of the function, as a Pydantic model.

          description: The description of the tool.

          tags: Metadata tags.

          source_type: The source type of the function.

          json_schema: The JSON schema of the function (auto-generated from source_code if not
              provided)

          return_char_limit: The maximum number of characters in the response.

          pip_requirements: Optional list of pip packages required by this tool.

          npm_requirements: Optional list of npm packages required by this tool.

          default_requires_approval: Whether or not to require approval before executing this tool.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

        Examples:
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        
        def add_two_numbers(a: int, b: int) -> int:
            return a + b
        
        await client.tools.create_from_function(
            func=add_two_numbers,
        )

        class InventoryEntryData(BaseModel):
            data: InventoryEntry
            quantity_change: int

        def manage_inventory(data: InventoryEntry, quantity_change: int) -> bool:
            pass
        
        await client.tools.create_from_function(
            func=manage_inventory,
            args_schema=InventoryEntryData,
        )
        """
        source_code = dedent(inspect.getsource(func))
        args_json_schema: Optional[Dict[str, object]] | Omit = omit
        if not isinstance(args_schema, Omit) and args_schema is not None:
            args_json_schema = args_schema.model_json_schema()

        return await self.create(
            source_code=source_code,
            args_json_schema=args_json_schema,
            description=description,
            tags=tags,
            source_type=source_type,
            json_schema=json_schema,
            return_char_limit=return_char_limit,
            pip_requirements=pip_requirements,
            npm_requirements=npm_requirements,
            default_requires_approval=default_requires_approval,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )


    async def upsert_from_function(
        self,
        *,
        func: typing.Callable[..., typing.Any],
        args_schema: typing.Optional[typing.Type[BaseModel]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        source_type: str | Omit = omit,
        json_schema: Optional[Dict[str, object]] | Omit = omit,
        return_char_limit: int | Omit = omit,
        pip_requirements: Optional[Iterable[PipRequirementParam]] | Omit = omit,
        npm_requirements: Optional[Iterable[NpmRequirementParam]] | Omit = omit,
        default_requires_approval: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Create or update a tool from a callable

        Args:
          func: The callable to create or update the tool from.

          args_schema: The arguments schema of the function, as a Pydantic model.

          description: The description of the tool.

          tags: Metadata tags.

          source_type: The source type of the function.

          json_schema: The JSON schema of the function (auto-generated from source_code if not
              provided)

          return_char_limit: The maximum number of characters in the response.

          pip_requirements: Optional list of pip packages required by this tool.

          npm_requirements: Optional list of npm packages required by this tool.

          default_requires_approval: Whether or not to require approval before executing this tool.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

        Examples:
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )

        def add_two_numbers(a: int, b: int) -> int:
            return a + b
        
        await client.tools.upsert_from_function(
            func=add_two_numbers,
        )

        class InventoryEntryData(BaseModel):
            data: InventoryEntry
            quantity_change: int

        def manage_inventory(data: InventoryEntry, quantity_change: int) -> bool:
            pass
        
        await client.tools.upsert_from_function(
            func=manage_inventory,
            args_schema=InventoryEntryData,
        )
        """
        source_code = dedent(inspect.getsource(func))
        args_json_schema: Optional[Dict[str, object]] | Omit = omit
        if not isinstance(args_schema, Omit) and args_schema is not None:
            args_json_schema = args_schema.model_json_schema()

        return await self.upsert(
            source_code=source_code,
            args_json_schema=args_json_schema,
            description=description,
            tags=tags,
            source_type=source_type,
            json_schema=json_schema,
            return_char_limit=return_char_limit,
            pip_requirements=pip_requirements,
            npm_requirements=npm_requirements,
            default_requires_approval=default_requires_approval,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
    
    async def add(
        self,
        *,
        tool: BaseTool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tool:
        """
        Add a tool to Letta from a custom Tool class

        Args:
          tool: The tool object to be added.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

        Examples:
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )

        class InventoryItem(BaseModel):
            sku: str  # Unique product identifier
            name: str  # Product name
            price: float  # Current price
            category: str  # Product category (e.g., "Electronics", "Clothing")

        class InventoryEntry(BaseModel):
            timestamp: int  # Unix timestamp of the transaction
            item: InventoryItem  # The product being updated
            transaction_id: str  # Unique identifier for this inventory update

        class InventoryEntryData(BaseModel):
            data: InventoryEntry
            quantity_change: int  # Change in quantity (positive for additions, negative for removals)

        class ManageInventoryTool(BaseTool):
            name: str = "manage_inventory"
            args_schema: Type[BaseModel] = InventoryEntryData
            description: str = "Update inventory catalogue with a new data entry"
            tags: List[str] = ["inventory", "shop"]

            def run(self, data: InventoryEntry, quantity_change: int) -> bool:
                '''
                Implementation of the manage_inventory tool
                '''
                print(f"Updated inventory for {data.item.name} with a quantity change of {quantity_change}")
                return True
                
        await client.tools.add(
            tool=ManageInventoryTool()
        )
        """
        source_code = tool.get_source_code()
        args_json_schema = tool.args_schema.model_json_schema() if tool.args_schema else None

        # Convert PipRequirement/NpmRequirement models to Param dicts
        pip_requirements_param = (
            [typing.cast(PipRequirementParam, req.model_dump()) for req in tool.pip_requirements]
            if tool.pip_requirements
            else omit
        )
        npm_requirements_param = (
            [typing.cast(NpmRequirementParam, req.model_dump()) for req in tool.npm_requirements]
            if tool.npm_requirements
            else omit
        )

        return await self.upsert(
            source_code=source_code,
            args_json_schema=args_json_schema or omit,
            description=tool.description or omit,
            tags=tool.tags or omit,
            source_type=tool.source_type or omit,
            json_schema=tool.json_schema or omit,
            return_char_limit=tool.return_char_limit or omit,
            pip_requirements=pip_requirements_param,
            npm_requirements=npm_requirements_param,
            default_requires_approval=tool.default_requires_approval or omit,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )


class ToolsResourceWithRawResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.create = to_raw_response_wrapper(
            tools.create,
        )
        self.retrieve = to_raw_response_wrapper(
            tools.retrieve,
        )
        self.update = to_raw_response_wrapper(
            tools.update,
        )
        self.list = to_raw_response_wrapper(
            tools.list,
        )
        self.delete = to_raw_response_wrapper(
            tools.delete,
        )
        self.upsert = to_raw_response_wrapper(
            tools.upsert,
        )
        self.upsert_base_tools = to_raw_response_wrapper(
            tools.upsert_base_tools,
        )


class AsyncToolsResourceWithRawResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.create = async_to_raw_response_wrapper(
            tools.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            tools.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            tools.update,
        )
        self.list = async_to_raw_response_wrapper(
            tools.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tools.delete,
        )
        self.upsert = async_to_raw_response_wrapper(
            tools.upsert,
        )
        self.upsert_base_tools = async_to_raw_response_wrapper(
            tools.upsert_base_tools,
        )


class ToolsResourceWithStreamingResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.create = to_streamed_response_wrapper(
            tools.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            tools.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            tools.update,
        )
        self.list = to_streamed_response_wrapper(
            tools.list,
        )
        self.delete = to_streamed_response_wrapper(
            tools.delete,
        )
        self.upsert = to_streamed_response_wrapper(
            tools.upsert,
        )
        self.upsert_base_tools = to_streamed_response_wrapper(
            tools.upsert_base_tools,
        )


class AsyncToolsResourceWithStreamingResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.create = async_to_streamed_response_wrapper(
            tools.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            tools.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            tools.update,
        )
        self.list = async_to_streamed_response_wrapper(
            tools.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tools.delete,
        )
        self.upsert = async_to_streamed_response_wrapper(
            tools.upsert,
        )
        self.upsert_base_tools = async_to_streamed_response_wrapper(
            tools.upsert_base_tools,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from .mcp_servers import (
    McpServersResource,
    AsyncMcpServersResource,
    McpServersResourceWithRawResponse,
    AsyncMcpServersResourceWithRawResponse,
    McpServersResourceWithStreamingResponse,
    AsyncMcpServersResourceWithStreamingResponse,
)

__all__ = [
    "ToolsResource",
    "AsyncToolsResource",
    "ToolsResourceWithRawResponse",
    "AsyncToolsResourceWithRawResponse",
    "ToolsResourceWithStreamingResponse",
    "AsyncToolsResourceWithStreamingResponse",
    "McpServersResource",
    "AsyncMcpServersResource",
    "McpServersResourceWithRawResponse",
    "AsyncMcpServersResourceWithRawResponse",
    "McpServersResourceWithStreamingResponse",
    "AsyncMcpServersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .._models import BaseModel
from .embedding_config import EmbeddingConfig
from .vector_db_provider import VectorDBProvider

__all__ = ["Archive"]


class Archive(BaseModel):
    id: str
    """The human-friendly ID of the Archive"""

    created_at: datetime
    """The creation date of the archive"""

    embedding_config: EmbeddingConfig
    """Embedding configuration for passages in this archive"""

    name: str
    """The name of the archive"""

    created_by_id: Optional[str] = None
    """The id of the user that made this object."""

    description: Optional[str] = None
    """A description of the archive"""

    last_updated_by_id: Optional[str] = None
    """The id of the user that made this object."""

    metadata: Optional[Dict[str, object]] = None
    """Additional metadata"""

    updated_at: Optional[datetime] = None
    """The timestamp when the object was last updated."""

    vector_db_provider: Optional[VectorDBProvider] = None
    """The vector database provider used for this archive's passages"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["AgentRetrieveParams"]


class AgentRetrieveParams(TypedDict, total=False):
    include: List[
        Literal[
            "agent.blocks",
            "agent.identities",
            "agent.managed_group",
            "agent.secrets",
            "agent.sources",
            "agent.tags",
            "agent.tools",
        ]
    ]
    """Specify which relational fields to include in the response.

    No relationships are included by default.
    """

    include_relationships: Optional[SequenceNotStr[str]]
    """
    Specify which relational fields (e.g., 'tools', 'sources', 'memory') to include
    in the response. If not provided, all relationships are loaded by default. Using
    this can optimize performance by reducing unnecessary joins.This is a legacy
    parameter, and no longer supported after 1.0.0 SDK versions.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["UsageRetrieveResponse", "CompletionTokensDetails", "PromptTokensDetails"]


class CompletionTokensDetails(BaseModel):
    reasoning_tokens: Optional[int] = None


class PromptTokensDetails(BaseModel):
    cached_tokens: Optional[int] = None


class UsageRetrieveResponse(BaseModel):
    completion_tokens: Optional[int] = None

    completion_tokens_details: Optional[CompletionTokensDetails] = None

    prompt_tokens: Optional[int] = None

    prompt_tokens_details: Optional[PromptTokensDetails] = None

    total_tokens: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ToolUpdateApprovalParams"]


class ToolUpdateApprovalParams(TypedDict, total=False):
    agent_id: Required[str]
    """The ID of the agent in the format 'agent-<uuid4>'"""

    body_requires_approval: Required[Annotated[bool, PropertyInfo(alias="requires_approval")]]
    """Whether the tool requires approval before execution"""

    query_requires_approval: Annotated[Optional[bool], PropertyInfo(alias="requires_approval")]
    """Whether the tool requires approval before execution"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .._models import BaseModel
from .embedding_config import EmbeddingConfig

__all__ = ["Folder"]


class Folder(BaseModel):
    id: str
    """The human-friendly ID of the Source"""

    embedding_config: EmbeddingConfig
    """The embedding configuration used by the folder."""

    name: str
    """The name of the folder."""

    created_at: Optional[datetime] = None
    """The timestamp when the folder was created."""

    created_by_id: Optional[str] = None
    """The id of the user that made this Tool."""

    description: Optional[str] = None
    """The description of the folder."""

    instructions: Optional[str] = None
    """Instructions for how to use the folder."""

    last_updated_by_id: Optional[str] = None
    """The id of the user that made this Tool."""

    metadata: Optional[Dict[str, object]] = None
    """Metadata associated with the folder."""

    updated_at: Optional[datetime] = None
    """The timestamp when the folder was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["CreateStdioMcpServerParam"]


class CreateStdioMcpServerParam(TypedDict, total=False):
    args: Required[SequenceNotStr[str]]
    """The arguments to pass to the command"""

    command: Required[str]
    """The command to run (MCP 'local' client will run this command)"""

    env: Optional[Dict[str, str]]
    """Environment variables to set"""

    mcp_server_type: Literal["stdio"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .passage_create_params import PassageCreateParams as PassageCreateParams
from .passage_create_response import PassageCreateResponse as PassageCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ToolReturnContentParam"]


class ToolReturnContentParam(TypedDict, total=False):
    content: Required[str]
    """The content returned by the tool execution."""

    is_error: Required[bool]
    """Indicates whether the tool execution resulted in an error."""

    tool_call_id: Required[str]
    """References the ID of the ToolCallContent that initiated this tool call."""

    type: Literal["tool_return"]
    """Indicates this content represents a tool return event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["MetricRetrieveResponse"]


class MetricRetrieveResponse(BaseModel):
    id: str
    """The id of the step this metric belongs to (matches steps.id)."""

    agent_id: Optional[str] = None
    """The unique identifier of the agent."""

    base_template_id: Optional[str] = None
    """The base template ID that the step belongs to (cloud only)."""

    llm_request_ns: Optional[int] = None
    """Time spent on LLM requests in nanoseconds."""

    llm_request_start_ns: Optional[int] = None
    """The timestamp of the start of the llm request in nanoseconds."""

    project_id: Optional[str] = None
    """The project that the step belongs to (cloud only)."""

    provider_id: Optional[str] = None
    """The unique identifier of the provider."""

    run_id: Optional[str] = None
    """The unique identifier of the run."""

    step_ns: Optional[int] = None
    """Total time for the step in nanoseconds."""

    step_start_ns: Optional[int] = None
    """The timestamp of the start of the step in nanoseconds."""

    template_id: Optional[str] = None
    """The template ID that the step belongs to (cloud only)."""

    tool_execution_ns: Optional[int] = None
    """Time spent on tool execution in nanoseconds."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["MessageStreamParams"]


class MessageStreamParams(TypedDict, total=False):
    batch_size: Optional[int]
    """Number of entries to read per batch."""

    include_pings: Optional[bool]
    """
    Whether to include periodic keepalive ping messages in the stream to prevent
    connection timeouts.
    """

    poll_interval: Optional[float]
    """Seconds to wait between polls when no new data."""

    starting_after: int
    """Sequence id to use as a cursor for pagination.

    Response will start streaming after this chunk sequence id
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ToolReturnParam"]


class ToolReturnParam(TypedDict, total=False):
    status: Required[Literal["success", "error"]]

    tool_call_id: Required[str]

    tool_return: Required[str]

    stderr: Optional[SequenceNotStr[str]]

    stdout: Optional[SequenceNotStr[str]]

    type: Literal["tool"]
    """The message type to be created."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

from .._types import SequenceNotStr
from .identity_type import IdentityType
from .identity_property_param import IdentityPropertyParam

__all__ = ["IdentityUpsertParams"]


class IdentityUpsertParams(TypedDict, total=False):
    identifier_key: Required[str]
    """External, user-generated identifier key of the identity."""

    identity_type: Required[IdentityType]
    """The type of the identity."""

    name: Required[str]
    """The name of the identity."""

    agent_ids: Optional[SequenceNotStr[str]]
    """The agent ids that are associated with the identity."""

    block_ids: Optional[SequenceNotStr[str]]
    """The IDs of the blocks associated with the identity."""

    project_id: Optional[str]
    """The project id of the identity, if applicable."""

    properties: Optional[Iterable[IdentityPropertyParam]]
    """List of properties associated with the identity."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RedactedReasoningContent"]


class RedactedReasoningContent(BaseModel):
    data: str
    """The redacted or filtered intermediate reasoning content."""

    type: Optional[Literal["redacted_reasoning"]] = None
    """Indicates this is a redacted thinking step."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...types import ManagerType, group_list_params, group_create_params, group_update_params
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncArrayPage, AsyncArrayPage
from ...types.group import Group
from ..._base_client import AsyncPaginator, make_request_options
from ...types.manager_type import ManagerType

__all__ = ["GroupsResource", "AsyncGroupsResource"]


class GroupsResource(SyncAPIResource):
    @cached_property
    def messages(self) -> MessagesResource:
        return MessagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> GroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return GroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return GroupsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        agent_ids: SequenceNotStr[str],
        description: str,
        hidden: Optional[bool] | Omit = omit,
        manager_config: group_create_params.ManagerConfig | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        shared_block_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Group:
        """
        Create a new multi-agent group with the specified configuration.

        Args:
          agent_ids

          description

          hidden: If set to True, the group will be hidden.

          manager_config

          project_id: The associated project id.

          shared_block_ids

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/groups/",
            body=maybe_transform(
                {
                    "agent_ids": agent_ids,
                    "description": description,
                    "hidden": hidden,
                    "manager_config": manager_config,
                    "project_id": project_id,
                    "shared_block_ids": shared_block_ids,
                },
                group_create_params.GroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Group,
        )

    def retrieve(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Group:
        """
        Retrieve the group by id.

        Args:
          group_id: The ID of the group in the format 'group-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return self._get(
            f"/v1/groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Group,
        )

    def update(
        self,
        group_id: str,
        *,
        agent_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        manager_config: Optional[group_update_params.ManagerConfig] | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        shared_block_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Group:
        """
        Create a new multi-agent group with the specified configuration.

        Args:
          group_id: The ID of the group in the format 'group-<uuid4>'

          agent_ids

          description

          manager_config

          project_id: The associated project id.

          shared_block_ids

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return self._patch(
            f"/v1/groups/{group_id}",
            body=maybe_transform(
                {
                    "agent_ids": agent_ids,
                    "description": description,
                    "manager_config": manager_config,
                    "project_id": project_id,
                    "shared_block_ids": shared_block_ids,
                },
                group_update_params.GroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Group,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        manager_type: Optional[ManagerType] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncArrayPage[Group]:
        """
        Fetch all multi-agent groups matching query.

        Args:
          after: Group ID cursor for pagination. Returns groups that come after this group ID in
              the specified sort order

          before: Group ID cursor for pagination. Returns groups that come before this group ID in
              the specified sort order

          limit: Maximum number of groups to return

          manager_type: Search groups by manager type

          order: Sort order for groups by creation time. 'asc' for oldest first, 'desc' for
              newest first

          order_by: Field to sort by

          project_id: Search groups by project id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/groups/",
            page=SyncArrayPage[Group],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "manager_type": manager_type,
                        "order": order,
                        "order_by": order_by,
                        "project_id": project_id,
                    },
                    group_list_params.GroupListParams,
                ),
            ),
            model=Group,
        )

    def delete(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a multi-agent group.

        Args:
          group_id: The ID of the group in the format 'group-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return self._delete(
            f"/v1/groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncGroupsResource(AsyncAPIResource):
    @cached_property
    def messages(self) -> AsyncMessagesResource:
        return AsyncMessagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncGroupsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        agent_ids: SequenceNotStr[str],
        description: str,
        hidden: Optional[bool] | Omit = omit,
        manager_config: group_create_params.ManagerConfig | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        shared_block_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Group:
        """
        Create a new multi-agent group with the specified configuration.

        Args:
          agent_ids

          description

          hidden: If set to True, the group will be hidden.

          manager_config

          project_id: The associated project id.

          shared_block_ids

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/groups/",
            body=await async_maybe_transform(
                {
                    "agent_ids": agent_ids,
                    "description": description,
                    "hidden": hidden,
                    "manager_config": manager_config,
                    "project_id": project_id,
                    "shared_block_ids": shared_block_ids,
                },
                group_create_params.GroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Group,
        )

    async def retrieve(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Group:
        """
        Retrieve the group by id.

        Args:
          group_id: The ID of the group in the format 'group-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return await self._get(
            f"/v1/groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Group,
        )

    async def update(
        self,
        group_id: str,
        *,
        agent_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        manager_config: Optional[group_update_params.ManagerConfig] | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        shared_block_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Group:
        """
        Create a new multi-agent group with the specified configuration.

        Args:
          group_id: The ID of the group in the format 'group-<uuid4>'

          agent_ids

          description

          manager_config

          project_id: The associated project id.

          shared_block_ids

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return await self._patch(
            f"/v1/groups/{group_id}",
            body=await async_maybe_transform(
                {
                    "agent_ids": agent_ids,
                    "description": description,
                    "manager_config": manager_config,
                    "project_id": project_id,
                    "shared_block_ids": shared_block_ids,
                },
                group_update_params.GroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Group,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        manager_type: Optional[ManagerType] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Group, AsyncArrayPage[Group]]:
        """
        Fetch all multi-agent groups matching query.

        Args:
          after: Group ID cursor for pagination. Returns groups that come after this group ID in
              the specified sort order

          before: Group ID cursor for pagination. Returns groups that come before this group ID in
              the specified sort order

          limit: Maximum number of groups to return

          manager_type: Search groups by manager type

          order: Sort order for groups by creation time. 'asc' for oldest first, 'desc' for
              newest first

          order_by: Field to sort by

          project_id: Search groups by project id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/groups/",
            page=AsyncArrayPage[Group],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "manager_type": manager_type,
                        "order": order,
                        "order_by": order_by,
                        "project_id": project_id,
                    },
                    group_list_params.GroupListParams,
                ),
            ),
            model=Group,
        )

    async def delete(
        self,
        group_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a multi-agent group.

        Args:
          group_id: The ID of the group in the format 'group-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return await self._delete(
            f"/v1/groups/{group_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class GroupsResourceWithRawResponse:
    def __init__(self, groups: GroupsResource) -> None:
        self._groups = groups

        self.create = to_raw_response_wrapper(
            groups.create,
        )
        self.retrieve = to_raw_response_wrapper(
            groups.retrieve,
        )
        self.update = to_raw_response_wrapper(
            groups.update,
        )
        self.list = to_raw_response_wrapper(
            groups.list,
        )
        self.delete = to_raw_response_wrapper(
            groups.delete,
        )

    @cached_property
    def messages(self) -> MessagesResourceWithRawResponse:
        return MessagesResourceWithRawResponse(self._groups.messages)


class AsyncGroupsResourceWithRawResponse:
    def __init__(self, groups: AsyncGroupsResource) -> None:
        self._groups = groups

        self.create = async_to_raw_response_wrapper(
            groups.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            groups.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            groups.update,
        )
        self.list = async_to_raw_response_wrapper(
            groups.list,
        )
        self.delete = async_to_raw_response_wrapper(
            groups.delete,
        )

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithRawResponse:
        return AsyncMessagesResourceWithRawResponse(self._groups.messages)


class GroupsResourceWithStreamingResponse:
    def __init__(self, groups: GroupsResource) -> None:
        self._groups = groups

        self.create = to_streamed_response_wrapper(
            groups.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            groups.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            groups.update,
        )
        self.list = to_streamed_response_wrapper(
            groups.list,
        )
        self.delete = to_streamed_response_wrapper(
            groups.delete,
        )

    @cached_property
    def messages(self) -> MessagesResourceWithStreamingResponse:
        return MessagesResourceWithStreamingResponse(self._groups.messages)


class AsyncGroupsResourceWithStreamingResponse:
    def __init__(self, groups: AsyncGroupsResource) -> None:
        self._groups = groups

        self.create = async_to_streamed_response_wrapper(
            groups.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            groups.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            groups.update,
        )
        self.list = async_to_streamed_response_wrapper(
            groups.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            groups.delete,
        )

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithStreamingResponse:
        return AsyncMessagesResourceWithStreamingResponse(self._groups.messages)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .tool import Tool

__all__ = ["ToolUpsertBaseToolsResponse"]

ToolUpsertBaseToolsResponse: TypeAlias = List[Tool]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .provider_type import ProviderType
from .provider_category import ProviderCategory

__all__ = ["Model"]


class Model(BaseModel):
    context_window: int
    """Deprecated: Use 'max_context_window' field instead.

    The context window size for the model.
    """

    max_context_window: int
    """The maximum context window for the model"""

    model: str
    """Deprecated: Use 'name' field instead. LLM model name."""

    api_model_endpoint_type: Literal[
        "openai",
        "anthropic",
        "google_ai",
        "google_vertex",
        "azure",
        "groq",
        "ollama",
        "webui",
        "webui-legacy",
        "lmstudio",
        "lmstudio-legacy",
        "lmstudio-chatcompletions",
        "llamacpp",
        "koboldcpp",
        "vllm",
        "hugging-face",
        "mistral",
        "together",
        "bedrock",
        "deepseek",
        "xai",
    ] = FieldInfo(alias="model_endpoint_type")
    """Deprecated: Use 'provider_type' field instead. The endpoint type for the model."""

    name: str
    """The actual model name used by the provider"""

    provider_type: ProviderType
    """The type of the provider"""

    compatibility_type: Optional[Literal["gguf", "mlx"]] = None
    """Deprecated: The framework compatibility type for the model."""

    display_name: Optional[str] = None
    """A human-friendly display name for the model."""

    enable_reasoner: Optional[bool] = None
    """
    Deprecated: Whether or not the model should use extended thinking if it is a
    'reasoning' style model.
    """

    frequency_penalty: Optional[float] = None
    """
    Deprecated: Positive values penalize new tokens based on their existing
    frequency in the text so far.
    """

    handle: Optional[str] = None
    """The handle for this config, in the format provider/model-name."""

    max_reasoning_tokens: Optional[int] = None
    """Deprecated: Configurable thinking budget for extended thinking."""

    max_tokens: Optional[int] = None
    """Deprecated: The maximum number of tokens to generate."""

    api_model_endpoint: Optional[str] = FieldInfo(alias="model_endpoint", default=None)
    """Deprecated: The endpoint for the model."""

    api_model_type: Optional[Literal["llm"]] = FieldInfo(alias="model_type", default=None)
    """Type of model (llm or embedding)"""

    api_model_wrapper: Optional[str] = FieldInfo(alias="model_wrapper", default=None)
    """Deprecated: The wrapper for the model."""

    parallel_tool_calls: Optional[bool] = None
    """Deprecated: If set to True, enables parallel tool calling."""

    provider_category: Optional[ProviderCategory] = None
    """Deprecated: The provider category for the model."""

    provider_name: Optional[str] = None
    """The provider name for the model."""

    put_inner_thoughts_in_kwargs: Optional[bool] = None
    """Deprecated: Puts 'inner_thoughts' as a kwarg in the function call."""

    reasoning_effort: Optional[Literal["none", "minimal", "low", "medium", "high"]] = None
    """Deprecated: The reasoning effort to use when generating text reasoning models."""

    temperature: Optional[float] = None
    """Deprecated: The temperature to use when generating text with the model."""

    tier: Optional[str] = None
    """Deprecated: The cost tier for the model (cloud only)."""

    verbosity: Optional[Literal["low", "medium", "high"]] = None
    """Deprecated: Soft control for how verbose model output should be."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .agents.tool_return import ToolReturn

__all__ = ["ToolReturnMessage"]


class ToolReturnMessage(BaseModel):
    id: str

    date: datetime

    status: Literal["success", "error"]

    tool_call_id: str

    tool_return: str

    is_err: Optional[bool] = None

    message_type: Optional[Literal["tool_return_message"]] = None
    """The type of the message."""

    name: Optional[str] = None

    otid: Optional[str] = None

    run_id: Optional[str] = None

    sender_id: Optional[str] = None

    seq_id: Optional[int] = None

    stderr: Optional[List[str]] = None

    stdout: Optional[List[str]] = None

    step_id: Optional[str] = None

    tool_returns: Optional[List[ToolReturn]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal

import httpx

from .files import (
    FilesResource,
    AsyncFilesResource,
    FilesResourceWithRawResponse,
    AsyncFilesResourceWithRawResponse,
    FilesResourceWithStreamingResponse,
    AsyncFilesResourceWithStreamingResponse,
)
from .agents import (
    AgentsResource,
    AsyncAgentsResource,
    AgentsResourceWithRawResponse,
    AsyncAgentsResourceWithRawResponse,
    AgentsResourceWithStreamingResponse,
    AsyncAgentsResourceWithStreamingResponse,
)
from ...types import folder_list_params, folder_create_params, folder_update_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncArrayPage, AsyncArrayPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.folder import Folder
from ...types.embedding_config_param import EmbeddingConfigParam

__all__ = ["FoldersResource", "AsyncFoldersResource"]


class FoldersResource(SyncAPIResource):
    @cached_property
    def files(self) -> FilesResource:
        return FilesResource(self._client)

    @cached_property
    def agents(self) -> AgentsResource:
        return AgentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> FoldersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return FoldersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FoldersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return FoldersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        description: Optional[str] | Omit = omit,
        embedding: Optional[str] | Omit = omit,
        embedding_chunk_size: Optional[int] | Omit = omit,
        embedding_config: Optional[EmbeddingConfigParam] | Omit = omit,
        instructions: Optional[str] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Folder:
        """
        Create a new data folder.

        Args:
          name: The name of the source.

          description: The description of the source.

          embedding: The handle for the embedding config used by the source.

          embedding_chunk_size: The chunk size of the embedding.

          embedding_config: Configuration for embedding model connection and processing parameters.

          instructions: Instructions for how to use the source.

          metadata: Metadata associated with the source.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/folders/",
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "embedding": embedding,
                    "embedding_chunk_size": embedding_chunk_size,
                    "embedding_config": embedding_config,
                    "instructions": instructions,
                    "metadata": metadata,
                },
                folder_create_params.FolderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Folder,
        )

    def retrieve(
        self,
        folder_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Folder:
        """
        Get a folder by ID

        Args:
          folder_id: The ID of the source in the format 'source-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not folder_id:
            raise ValueError(f"Expected a non-empty value for `folder_id` but received {folder_id!r}")
        return self._get(
            f"/v1/folders/{folder_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Folder,
        )

    def update(
        self,
        folder_id: str,
        *,
        description: Optional[str] | Omit = omit,
        embedding_config: Optional[EmbeddingConfigParam] | Omit = omit,
        instructions: Optional[str] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Folder:
        """
        Update the name or documentation of an existing data folder.

        Args:
          folder_id: The ID of the source in the format 'source-<uuid4>'

          description: The description of the source.

          embedding_config: Configuration for embedding model connection and processing parameters.

          instructions: Instructions for how to use the source.

          metadata: Metadata associated with the source.

          name: The name of the source.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not folder_id:
            raise ValueError(f"Expected a non-empty value for `folder_id` but received {folder_id!r}")
        return self._patch(
            f"/v1/folders/{folder_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "embedding_config": embedding_config,
                    "instructions": instructions,
                    "metadata": metadata,
                    "name": name,
                },
                folder_update_params.FolderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Folder,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        name: Optional[str] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncArrayPage[Folder]:
        """
        List all data folders created by a user.

        Args:
          after: Folder ID cursor for pagination. Returns folders that come after this folder ID
              in the specified sort order

          before: Folder ID cursor for pagination. Returns folders that come before this folder ID
              in the specified sort order

          limit: Maximum number of folders to return

          name: Folder name to filter by

          order: Sort order for folders by creation time. 'asc' for oldest first, 'desc' for
              newest first

          order_by: Field to sort by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/folders/",
            page=SyncArrayPage[Folder],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "name": name,
                        "order": order,
                        "order_by": order_by,
                    },
                    folder_list_params.FolderListParams,
                ),
            ),
            model=Folder,
        )

    def delete(
        self,
        folder_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a data folder.

        Args:
          folder_id: The ID of the source in the format 'source-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not folder_id:
            raise ValueError(f"Expected a non-empty value for `folder_id` but received {folder_id!r}")
        return self._delete(
            f"/v1/folders/{folder_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncFoldersResource(AsyncAPIResource):
    @cached_property
    def files(self) -> AsyncFilesResource:
        return AsyncFilesResource(self._client)

    @cached_property
    def agents(self) -> AsyncAgentsResource:
        return AsyncAgentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFoldersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFoldersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFoldersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncFoldersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        description: Optional[str] | Omit = omit,
        embedding: Optional[str] | Omit = omit,
        embedding_chunk_size: Optional[int] | Omit = omit,
        embedding_config: Optional[EmbeddingConfigParam] | Omit = omit,
        instructions: Optional[str] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Folder:
        """
        Create a new data folder.

        Args:
          name: The name of the source.

          description: The description of the source.

          embedding: The handle for the embedding config used by the source.

          embedding_chunk_size: The chunk size of the embedding.

          embedding_config: Configuration for embedding model connection and processing parameters.

          instructions: Instructions for how to use the source.

          metadata: Metadata associated with the source.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/folders/",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "embedding": embedding,
                    "embedding_chunk_size": embedding_chunk_size,
                    "embedding_config": embedding_config,
                    "instructions": instructions,
                    "metadata": metadata,
                },
                folder_create_params.FolderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Folder,
        )

    async def retrieve(
        self,
        folder_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Folder:
        """
        Get a folder by ID

        Args:
          folder_id: The ID of the source in the format 'source-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not folder_id:
            raise ValueError(f"Expected a non-empty value for `folder_id` but received {folder_id!r}")
        return await self._get(
            f"/v1/folders/{folder_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Folder,
        )

    async def update(
        self,
        folder_id: str,
        *,
        description: Optional[str] | Omit = omit,
        embedding_config: Optional[EmbeddingConfigParam] | Omit = omit,
        instructions: Optional[str] | Omit = omit,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Folder:
        """
        Update the name or documentation of an existing data folder.

        Args:
          folder_id: The ID of the source in the format 'source-<uuid4>'

          description: The description of the source.

          embedding_config: Configuration for embedding model connection and processing parameters.

          instructions: Instructions for how to use the source.

          metadata: Metadata associated with the source.

          name: The name of the source.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not folder_id:
            raise ValueError(f"Expected a non-empty value for `folder_id` but received {folder_id!r}")
        return await self._patch(
            f"/v1/folders/{folder_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "embedding_config": embedding_config,
                    "instructions": instructions,
                    "metadata": metadata,
                    "name": name,
                },
                folder_update_params.FolderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Folder,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        name: Optional[str] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Folder, AsyncArrayPage[Folder]]:
        """
        List all data folders created by a user.

        Args:
          after: Folder ID cursor for pagination. Returns folders that come after this folder ID
              in the specified sort order

          before: Folder ID cursor for pagination. Returns folders that come before this folder ID
              in the specified sort order

          limit: Maximum number of folders to return

          name: Folder name to filter by

          order: Sort order for folders by creation time. 'asc' for oldest first, 'desc' for
              newest first

          order_by: Field to sort by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/folders/",
            page=AsyncArrayPage[Folder],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "name": name,
                        "order": order,
                        "order_by": order_by,
                    },
                    folder_list_params.FolderListParams,
                ),
            ),
            model=Folder,
        )

    async def delete(
        self,
        folder_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a data folder.

        Args:
          folder_id: The ID of the source in the format 'source-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not folder_id:
            raise ValueError(f"Expected a non-empty value for `folder_id` but received {folder_id!r}")
        return await self._delete(
            f"/v1/folders/{folder_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class FoldersResourceWithRawResponse:
    def __init__(self, folders: FoldersResource) -> None:
        self._folders = folders

        self.create = to_raw_response_wrapper(
            folders.create,
        )
        self.retrieve = to_raw_response_wrapper(
            folders.retrieve,
        )
        self.update = to_raw_response_wrapper(
            folders.update,
        )
        self.list = to_raw_response_wrapper(
            folders.list,
        )
        self.delete = to_raw_response_wrapper(
            folders.delete,
        )

    @cached_property
    def files(self) -> FilesResourceWithRawResponse:
        return FilesResourceWithRawResponse(self._folders.files)

    @cached_property
    def agents(self) -> AgentsResourceWithRawResponse:
        return AgentsResourceWithRawResponse(self._folders.agents)


class AsyncFoldersResourceWithRawResponse:
    def __init__(self, folders: AsyncFoldersResource) -> None:
        self._folders = folders

        self.create = async_to_raw_response_wrapper(
            folders.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            folders.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            folders.update,
        )
        self.list = async_to_raw_response_wrapper(
            folders.list,
        )
        self.delete = async_to_raw_response_wrapper(
            folders.delete,
        )

    @cached_property
    def files(self) -> AsyncFilesResourceWithRawResponse:
        return AsyncFilesResourceWithRawResponse(self._folders.files)

    @cached_property
    def agents(self) -> AsyncAgentsResourceWithRawResponse:
        return AsyncAgentsResourceWithRawResponse(self._folders.agents)


class FoldersResourceWithStreamingResponse:
    def __init__(self, folders: FoldersResource) -> None:
        self._folders = folders

        self.create = to_streamed_response_wrapper(
            folders.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            folders.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            folders.update,
        )
        self.list = to_streamed_response_wrapper(
            folders.list,
        )
        self.delete = to_streamed_response_wrapper(
            folders.delete,
        )

    @cached_property
    def files(self) -> FilesResourceWithStreamingResponse:
        return FilesResourceWithStreamingResponse(self._folders.files)

    @cached_property
    def agents(self) -> AgentsResourceWithStreamingResponse:
        return AgentsResourceWithStreamingResponse(self._folders.agents)


class AsyncFoldersResourceWithStreamingResponse:
    def __init__(self, folders: AsyncFoldersResource) -> None:
        self._folders = folders

        self.create = async_to_streamed_response_wrapper(
            folders.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            folders.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            folders.update,
        )
        self.list = async_to_streamed_response_wrapper(
            folders.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            folders.delete,
        )

    @cached_property
    def files(self) -> AsyncFilesResourceWithStreamingResponse:
        return AsyncFilesResourceWithStreamingResponse(self._folders.files)

    @cached_property
    def agents(self) -> AsyncAgentsResourceWithStreamingResponse:
        return AsyncAgentsResourceWithStreamingResponse(self._folders.agents)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .agent_list_params import AgentListParams as AgentListParams
from .block_list_params import BlockListParams as BlockListParams
from .property_upsert_params import PropertyUpsertParams as PropertyUpsertParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .provider_category import ProviderCategory

__all__ = ["LlmConfig"]


class LlmConfig(BaseModel):
    context_window: int
    """The context window size for the model."""

    model: str
    """LLM model name."""

    api_model_endpoint_type: Literal[
        "openai",
        "anthropic",
        "google_ai",
        "google_vertex",
        "azure",
        "groq",
        "ollama",
        "webui",
        "webui-legacy",
        "lmstudio",
        "lmstudio-legacy",
        "lmstudio-chatcompletions",
        "llamacpp",
        "koboldcpp",
        "vllm",
        "hugging-face",
        "mistral",
        "together",
        "bedrock",
        "deepseek",
        "xai",
    ] = FieldInfo(alias="model_endpoint_type")
    """The endpoint type for the model."""

    compatibility_type: Optional[Literal["gguf", "mlx"]] = None
    """The framework compatibility type for the model."""

    display_name: Optional[str] = None
    """A human-friendly display name for the model."""

    enable_reasoner: Optional[bool] = None
    """
    Whether or not the model should use extended thinking if it is a 'reasoning'
    style model
    """

    frequency_penalty: Optional[float] = None
    """
    Positive values penalize new tokens based on their existing frequency in the
    text so far, decreasing the model's likelihood to repeat the same line verbatim.
    From OpenAI: Number between -2.0 and 2.0.
    """

    handle: Optional[str] = None
    """The handle for this config, in the format provider/model-name."""

    max_reasoning_tokens: Optional[int] = None
    """Configurable thinking budget for extended thinking.

    Used for enable_reasoner and also for Google Vertex models like Gemini 2.5
    Flash. Minimum value is 1024 when used with enable_reasoner.
    """

    max_tokens: Optional[int] = None
    """The maximum number of tokens to generate.

    If not set, the model will use its default value.
    """

    api_model_endpoint: Optional[str] = FieldInfo(alias="model_endpoint", default=None)
    """The endpoint for the model."""

    api_model_wrapper: Optional[str] = FieldInfo(alias="model_wrapper", default=None)
    """The wrapper for the model."""

    parallel_tool_calls: Optional[bool] = None
    """Deprecated: Use model_settings to configure parallel tool calls instead.

    If set to True, enables parallel tool calling. Defaults to False.
    """

    provider_category: Optional[ProviderCategory] = None
    """The provider category for the model."""

    provider_name: Optional[str] = None
    """The provider name for the model."""

    put_inner_thoughts_in_kwargs: Optional[bool] = None
    """Puts 'inner_thoughts' as a kwarg in the function call if this is set to True.

    This helps with function calling performance and also the generation of inner
    thoughts.
    """

    reasoning_effort: Optional[Literal["none", "minimal", "low", "medium", "high"]] = None
    """The reasoning effort to use when generating text reasoning models"""

    temperature: Optional[float] = None
    """The temperature to use when generating text with the model.

    A higher temperature will result in more random text.
    """

    tier: Optional[str] = None
    """The cost tier for the model (cloud only)."""

    verbosity: Optional[Literal["low", "medium", "high"]] = None
    """Soft control for how verbose model output should be, used for GPT-5 models."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .agents import (
    AgentsResource,
    AsyncAgentsResource,
    AgentsResourceWithRawResponse,
    AsyncAgentsResourceWithRawResponse,
    AgentsResourceWithStreamingResponse,
    AsyncAgentsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["TemplatesResource", "AsyncTemplatesResource"]


class TemplatesResource(SyncAPIResource):
    @cached_property
    def agents(self) -> AgentsResource:
        return AgentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> TemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return TemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return TemplatesResourceWithStreamingResponse(self)


class AsyncTemplatesResource(AsyncAPIResource):
    @cached_property
    def agents(self) -> AsyncAgentsResource:
        return AsyncAgentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncTemplatesResourceWithStreamingResponse(self)


class TemplatesResourceWithRawResponse:
    def __init__(self, templates: TemplatesResource) -> None:
        self._templates = templates

    @cached_property
    def agents(self) -> AgentsResourceWithRawResponse:
        return AgentsResourceWithRawResponse(self._templates.agents)


class AsyncTemplatesResourceWithRawResponse:
    def __init__(self, templates: AsyncTemplatesResource) -> None:
        self._templates = templates

    @cached_property
    def agents(self) -> AsyncAgentsResourceWithRawResponse:
        return AsyncAgentsResourceWithRawResponse(self._templates.agents)


class TemplatesResourceWithStreamingResponse:
    def __init__(self, templates: TemplatesResource) -> None:
        self._templates = templates

    @cached_property
    def agents(self) -> AgentsResourceWithStreamingResponse:
        return AgentsResourceWithStreamingResponse(self._templates.agents)


class AsyncTemplatesResourceWithStreamingResponse:
    def __init__(self, templates: AsyncTemplatesResource) -> None:
        self._templates = templates

    @cached_property
    def agents(self) -> AsyncAgentsResourceWithStreamingResponse:
        return AsyncAgentsResourceWithStreamingResponse(self._templates.agents)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from ..._types import FileTypes

__all__ = ["FileUploadParams"]


class FileUploadParams(TypedDict, total=False):
    file: Required[FileTypes]

    duplicate_handling: Literal["skip", "error", "suffix", "replace"]
    """How to handle duplicate filenames"""

    name: Optional[str]
    """Optional custom name to override the uploaded file's name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["OmittedReasoningContentParam"]


class OmittedReasoningContentParam(TypedDict, total=False):
    signature: Optional[str]
    """A unique identifier for this reasoning step."""

    type: Literal["omitted_reasoning"]
    """Indicates this is an omitted reasoning step."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .manager_type import ManagerType

__all__ = ["Group"]


class Group(BaseModel):
    id: str
    """The id of the group. Assigned by the database."""

    agent_ids: List[str]

    description: str

    manager_type: ManagerType

    base_template_id: Optional[str] = None
    """The base template id."""

    deployment_id: Optional[str] = None
    """The id of the deployment."""

    hidden: Optional[bool] = None
    """If set to True, the group will be hidden."""

    last_processed_message_id: Optional[str] = None

    manager_agent_id: Optional[str] = None

    max_message_buffer_length: Optional[int] = None
    """The desired maximum length of messages in the context window of the convo agent.

    This is a best effort, and may be off slightly due to user/assistant
    interleaving.
    """

    max_turns: Optional[int] = None

    min_message_buffer_length: Optional[int] = None
    """The desired minimum length of messages in the context window of the convo agent.

    This is a best effort, and may be off-by-one due to user/assistant interleaving.
    """

    project_id: Optional[str] = None
    """The associated project id."""

    shared_block_ids: Optional[List[str]] = None

    sleeptime_agent_frequency: Optional[int] = None

    template_id: Optional[str] = None
    """The id of the template."""

    termination_token: Optional[str] = None

    turns_counter: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["AnthropicModelSettingsParam", "Thinking"]


class Thinking(TypedDict, total=False):
    budget_tokens: int
    """The maximum number of tokens the model can use for extended thinking."""

    type: Literal["enabled", "disabled"]
    """The type of thinking to use."""


class AnthropicModelSettingsParam(TypedDict, total=False):
    max_output_tokens: int
    """The maximum number of tokens the model can generate."""

    parallel_tool_calls: bool
    """Whether to enable parallel tool calling."""

    provider_type: Literal["anthropic"]
    """The type of the provider."""

    temperature: float
    """The temperature of the model."""

    thinking: Thinking
    """The thinking configuration for the model."""

    verbosity: Optional[Literal["low", "medium", "high"]]
    """Soft control for how verbose model output should be, used for GPT-5 models."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .message_type import MessageType
from ..stop_reason_type import StopReasonType

__all__ = ["Run", "RequestConfig"]


class RequestConfig(BaseModel):
    assistant_message_tool_kwarg: Optional[str] = None
    """The name of the message argument in the designated message tool."""

    assistant_message_tool_name: Optional[str] = None
    """The name of the designated message tool."""

    include_return_message_types: Optional[List[MessageType]] = None
    """Only return specified message types in the response.

    If `None` (default) returns all messages.
    """

    use_assistant_message: Optional[bool] = None
    """
    Whether the server should parse specific tool call arguments (default
    `send_message`) as `AssistantMessage` objects.
    """


class Run(BaseModel):
    id: str
    """The human-friendly ID of the Run"""

    agent_id: str
    """The unique identifier of the agent associated with the run."""

    background: Optional[bool] = None
    """Whether the run was created in background mode."""

    base_template_id: Optional[str] = None
    """The base template ID that the run belongs to."""

    callback_error: Optional[str] = None
    """Optional error message from attempting to POST the callback endpoint."""

    callback_sent_at: Optional[datetime] = None
    """Timestamp when the callback was last attempted."""

    callback_status_code: Optional[int] = None
    """HTTP status code returned by the callback endpoint."""

    callback_url: Optional[str] = None
    """If set, POST to this URL when the run completes."""

    completed_at: Optional[datetime] = None
    """The timestamp when the run was completed."""

    created_at: Optional[datetime] = None
    """The timestamp when the run was created."""

    metadata: Optional[Dict[str, object]] = None
    """Additional metadata for the run."""

    request_config: Optional[RequestConfig] = None
    """The request configuration for the run."""

    status: Optional[Literal["created", "running", "completed", "failed", "cancelled"]] = None
    """The current status of the run."""

    stop_reason: Optional[StopReasonType] = None
    """The reason why the run was stopped."""

    total_duration_ns: Optional[int] = None
    """Total run duration in nanoseconds"""

    ttft_ns: Optional[int] = None
    """Time to first token for a run in nanoseconds"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..agents.user_message import UserMessage
from ..tool_return_message import ToolReturnMessage
from ..agents.event_message import EventMessage
from ..agents.system_message import SystemMessage
from ..agents.summary_message import SummaryMessage
from ..agents.assistant_message import AssistantMessage
from ..agents.reasoning_message import ReasoningMessage
from ..agents.tool_call_message import ToolCallMessage
from ..agents.approval_request_message import ApprovalRequestMessage
from ..agents.hidden_reasoning_message import HiddenReasoningMessage
from ..agents.approval_response_message import ApprovalResponseMessage

__all__ = ["MessageListResponse"]

MessageListResponse: TypeAlias = Annotated[
    Union[
        SystemMessage,
        UserMessage,
        ReasoningMessage,
        HiddenReasoningMessage,
        ToolCallMessage,
        ToolReturnMessage,
        AssistantMessage,
        ApprovalRequestMessage,
        ApprovalResponseMessage,
        SummaryMessage,
        EventMessage,
    ],
    PropertyInfo(discriminator="message_type"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["ParentToolRuleParam"]


class ParentToolRuleParam(TypedDict, total=False):
    children: Required[SequenceNotStr[str]]
    """The children tools that can be invoked."""

    tool_name: Required[str]
    """The name of the tool. Must exist in the database for the user's organization."""

    prompt_template: Optional[str]
    """Optional template string (ignored)."""

    type: Literal["parent_last_tool"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["ProviderTrace"]


class ProviderTrace(BaseModel):
    request_json: Dict[str, object]
    """JSON content of the provider request"""

    response_json: Dict[str, object]
    """JSON content of the provider response"""

    id: Optional[str] = None
    """The human-friendly ID of the Provider_trace"""

    created_at: Optional[datetime] = None
    """The timestamp when the object was created."""

    created_by_id: Optional[str] = None
    """The id of the user that made this object."""

    last_updated_by_id: Optional[str] = None
    """The id of the user that made this object."""

    step_id: Optional[str] = None
    """ID of the step that this trace is associated with"""

    updated_at: Optional[datetime] = None
    """The timestamp when the object was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["DynamicManagerParam"]


class DynamicManagerParam(TypedDict, total=False):
    manager_agent_id: Required[str]

    manager_type: Literal["dynamic"]

    max_turns: Optional[int]

    termination_token: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["JsonSchemaResponseFormat"]


class JsonSchemaResponseFormat(BaseModel):
    json_schema: Dict[str, object]
    """The JSON schema of the response."""

    type: Optional[Literal["json_schema"]] = None
    """The type of the response format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = [
    "GroupUpdateParams",
    "ManagerConfig",
    "ManagerConfigRoundRobinManagerUpdate",
    "ManagerConfigSupervisorManagerUpdate",
    "ManagerConfigDynamicManagerUpdate",
    "ManagerConfigSleeptimeManagerUpdate",
    "ManagerConfigVoiceSleeptimeManagerUpdate",
]


class GroupUpdateParams(TypedDict, total=False):
    agent_ids: Optional[SequenceNotStr[str]]

    description: Optional[str]

    manager_config: Optional[ManagerConfig]

    project_id: Optional[str]
    """The associated project id."""

    shared_block_ids: Optional[SequenceNotStr[str]]


class ManagerConfigRoundRobinManagerUpdate(TypedDict, total=False):
    manager_type: Literal["round_robin"]

    max_turns: Optional[int]


class ManagerConfigSupervisorManagerUpdate(TypedDict, total=False):
    manager_agent_id: Required[Optional[str]]

    manager_type: Literal["supervisor"]


class ManagerConfigDynamicManagerUpdate(TypedDict, total=False):
    manager_agent_id: Optional[str]

    manager_type: Literal["dynamic"]

    max_turns: Optional[int]

    termination_token: Optional[str]


class ManagerConfigSleeptimeManagerUpdate(TypedDict, total=False):
    manager_agent_id: Optional[str]

    manager_type: Literal["sleeptime"]

    sleeptime_agent_frequency: Optional[int]


class ManagerConfigVoiceSleeptimeManagerUpdate(TypedDict, total=False):
    manager_agent_id: Optional[str]

    manager_type: Literal["voice_sleeptime"]

    max_message_buffer_length: Optional[int]
    """The desired maximum length of messages in the context window of the convo agent.

    This is a best effort, and may be off slightly due to user/assistant
    interleaving.
    """

    min_message_buffer_length: Optional[int]
    """The desired minimum length of messages in the context window of the convo agent.

    This is a best effort, and may be off-by-one due to user/assistant interleaving.
    """


ManagerConfig: TypeAlias = Union[
    ManagerConfigRoundRobinManagerUpdate,
    ManagerConfigSupervisorManagerUpdate,
    ManagerConfigDynamicManagerUpdate,
    ManagerConfigSleeptimeManagerUpdate,
    ManagerConfigVoiceSleeptimeManagerUpdate,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .files import (
    FilesResource,
    AsyncFilesResource,
    FilesResourceWithRawResponse,
    AsyncFilesResourceWithRawResponse,
    FilesResourceWithStreamingResponse,
    AsyncFilesResourceWithStreamingResponse,
)
from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from .agents import (
    AgentsResource,
    AsyncAgentsResource,
    AgentsResourceWithRawResponse,
    AsyncAgentsResourceWithRawResponse,
    AgentsResourceWithStreamingResponse,
    AsyncAgentsResourceWithStreamingResponse,
)
from .blocks import (
    BlocksResource,
    AsyncBlocksResource,
    BlocksResourceWithRawResponse,
    AsyncBlocksResourceWithRawResponse,
    BlocksResourceWithStreamingResponse,
    AsyncBlocksResourceWithStreamingResponse,
)
from .groups import (
    GroupsResource,
    AsyncGroupsResource,
    GroupsResourceWithRawResponse,
    AsyncGroupsResourceWithRawResponse,
    GroupsResourceWithStreamingResponse,
    AsyncGroupsResourceWithStreamingResponse,
)
from .folders import (
    FoldersResource,
    AsyncFoldersResource,
    FoldersResourceWithRawResponse,
    AsyncFoldersResourceWithRawResponse,
    FoldersResourceWithStreamingResponse,
    AsyncFoldersResourceWithStreamingResponse,
)
from .archives import (
    ArchivesResource,
    AsyncArchivesResource,
    ArchivesResourceWithRawResponse,
    AsyncArchivesResourceWithRawResponse,
    ArchivesResourceWithStreamingResponse,
    AsyncArchivesResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from .identities import (
    IdentitiesResource,
    AsyncIdentitiesResource,
    IdentitiesResourceWithRawResponse,
    AsyncIdentitiesResourceWithRawResponse,
    IdentitiesResourceWithStreamingResponse,
    AsyncIdentitiesResourceWithStreamingResponse,
)

__all__ = [
    "MessagesResource",
    "AsyncMessagesResource",
    "MessagesResourceWithRawResponse",
    "AsyncMessagesResourceWithRawResponse",
    "MessagesResourceWithStreamingResponse",
    "AsyncMessagesResourceWithStreamingResponse",
    "BlocksResource",
    "AsyncBlocksResource",
    "BlocksResourceWithRawResponse",
    "AsyncBlocksResourceWithRawResponse",
    "BlocksResourceWithStreamingResponse",
    "AsyncBlocksResourceWithStreamingResponse",
    "ToolsResource",
    "AsyncToolsResource",
    "ToolsResourceWithRawResponse",
    "AsyncToolsResourceWithRawResponse",
    "ToolsResourceWithStreamingResponse",
    "AsyncToolsResourceWithStreamingResponse",
    "FoldersResource",
    "AsyncFoldersResource",
    "FoldersResourceWithRawResponse",
    "AsyncFoldersResourceWithRawResponse",
    "FoldersResourceWithStreamingResponse",
    "AsyncFoldersResourceWithStreamingResponse",
    "FilesResource",
    "AsyncFilesResource",
    "FilesResourceWithRawResponse",
    "AsyncFilesResourceWithRawResponse",
    "FilesResourceWithStreamingResponse",
    "AsyncFilesResourceWithStreamingResponse",
    "GroupsResource",
    "AsyncGroupsResource",
    "GroupsResourceWithRawResponse",
    "AsyncGroupsResourceWithRawResponse",
    "GroupsResourceWithStreamingResponse",
    "AsyncGroupsResourceWithStreamingResponse",
    "ArchivesResource",
    "AsyncArchivesResource",
    "ArchivesResourceWithRawResponse",
    "AsyncArchivesResourceWithRawResponse",
    "ArchivesResourceWithStreamingResponse",
    "AsyncArchivesResourceWithStreamingResponse",
    "IdentitiesResource",
    "AsyncIdentitiesResource",
    "IdentitiesResourceWithRawResponse",
    "AsyncIdentitiesResourceWithRawResponse",
    "IdentitiesResourceWithStreamingResponse",
    "AsyncIdentitiesResourceWithStreamingResponse",
    "AgentsResource",
    "AsyncAgentsResource",
    "AgentsResourceWithRawResponse",
    "AsyncAgentsResourceWithRawResponse",
    "AgentsResourceWithStreamingResponse",
    "AsyncAgentsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from . import tool_call
from ..._models import BaseModel
from .tool_call_delta import ToolCallDelta

__all__ = ["ToolCallMessage", "ToolCall", "ToolCalls"]

ToolCall: TypeAlias = Union[tool_call.ToolCall, ToolCallDelta]

ToolCalls: TypeAlias = Union[List[tool_call.ToolCall], ToolCallDelta, None]


class ToolCallMessage(BaseModel):
    id: str

    date: datetime

    tool_call: ToolCall

    is_err: Optional[bool] = None

    message_type: Optional[Literal["tool_call_message"]] = None
    """The type of the message."""

    name: Optional[str] = None

    otid: Optional[str] = None

    run_id: Optional[str] = None

    sender_id: Optional[str] = None

    seq_id: Optional[int] = None

    step_id: Optional[str] = None

    tool_calls: Optional[ToolCalls] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ReasoningContentParam"]


class ReasoningContentParam(TypedDict, total=False):
    is_native: Required[bool]
    """
    Whether the reasoning content was generated by a reasoner model that processed
    this step.
    """

    reasoning: Required[str]
    """The intermediate reasoning or thought process content."""

    signature: Optional[str]
    """A unique identifier for this reasoning step."""

    type: Literal["reasoning"]
    """Indicates this is a reasoning/intermediate step."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["MessageType"]

MessageType: TypeAlias = Literal[
    "system_message",
    "user_message",
    "assistant_message",
    "reasoning_message",
    "hidden_reasoning_message",
    "tool_call_message",
    "tool_return_message",
    "approval_request_message",
    "approval_response_message",
]
